@@ -175,10 +175,10 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
   ### 2. Search initial grid before optimization  ####
   if (grid_search) {
     if (logging==TRUE) {
-      log_info(paste(length(inits[,1]), "...parameter sets to check"))
-      log_info(paste("data got ", nrow(df), " rows"))
+      logger::log_info(paste(length(inits[,1]), "...parameter sets to check"))
+      logger::log_info(paste("data got ", nrow(df), " rows"))
       t00 <- Sys.time()
-      log_info("Searching initial values ...")
+      logger::log_info("Searching initial values ...")
     }
 
     if (optim_method =="Nelder-Mead") {
@@ -212,7 +212,7 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
     logL <- as.numeric(logL)
     inits <- inits[order(logL),]
     if (logging==TRUE) {
-      log_success(paste("Initial grid search took...",as.character(round(as.double(difftime(Sys.time(),t00,units = "mins")), 2))," mins"))
+      logger::log_success(paste("Initial grid search took...",as.character(round(as.double(difftime(Sys.time(),t00,units = "mins")), 2))," mins"))
     }
   } else {
     logL <- NULL
@@ -227,7 +227,7 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
 
   #### 3. Optimization ####
   if (logging==TRUE) {
-    log_info("Start fitting ... ")
+    logger::log_info("Start fitting ... ")
   }
   if (!useparallel || (opts$nAttempts==1)) {
     noFitYet <- TRUE
@@ -272,12 +272,12 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
           stop(paste("Not implemented or unknown method: ", optim_method, ". Use 'bobyqa', Nelder-Mead' or 'L-BFGS-B' instead.", sep=""))
         }
         if (logging==TRUE) {
-          log_info(paste("Finished attempt No.", i, " restart no. ", l))
+          logger::log_info(paste("Finished attempt No.", i, " restart no. ", l))
         }
         if (!exists("m") || inherits(m, "try-error")){
           if (logging==TRUE) {
-            log_error(paste("No fit obtained at attempt No.", i))
-            log_error(paste("Used parameter set", paste(start, sep="", collapse=" "), sep=" ", collapse = ""))
+            logger::log_error(paste("No fit obtained at attempt No.", i))
+            logger::log_error(paste("Used parameter set", paste(start, sep="", collapse=" "), sep=" ", collapse = ""))
           }
           break
         }
@@ -286,7 +286,7 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
             fit <- m
             noFitYet <- FALSE
             if (logging==TRUE) {
-              log_info(paste("First fit obtained at attempt No.", i))
+              logger::log_info(paste("First fit obtained at attempt No.", i))
               attempt <- i
               save(logL, inits,  df,fit, attempt,file=filename)
             }
@@ -294,7 +294,7 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
           } else if (m$value < fit$value) {
             fit <- m
             if (logging==TRUE) {
-              log_info(paste("New fit at attempt No.", i, " restart no. ", l))
+              logger::log_info(paste("New fit at attempt No.", i, " restart no. ", l))
               attempt <- i
               save(logL, inits,  df,fit, attempt,file=filename)
             }
@@ -485,7 +485,7 @@ fittingPCRM <- function(df, nConds, nRatings, fixed, sym_thetas, time_scaled,
     res$AICc <- 2 * fit$value + k * 2 + 2*k*(k-1)/(N-k-1)
     res$AIC <- 2 * fit$value + k * 2
     if (logging==TRUE) {
-      log_success("Done fitting and autosaved results")
+      logger::log_success("Done fitting and autosaved results")
       save(logL, df, res, file=filename)
     }
   }
