@@ -16,42 +16,38 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
     # response time = decision time + interjudgment time (tau)
     #                  + non-judgment time (t0)
     simult_conf = TRUE
-    restr_tau = mint0
+    restr_tau = 1
   }
   ### 1. Generate initial grid for grid search over possible parameter sets ####
   #### Create grid ####
   if (is.null(init_grid)) {
     if (restr_tau == Inf) {
-      tau = mint0*seq(0.2, 0.9, length.out = 3)
+      tau = mint0*seq(0.8, 1.8, length.out = 3)
     } else if (simult_conf) {
-      tau = mint0*seq(0.1, 0.4, length.out = 3)
+      tau = seq(0.1, 0.8, length.out = 3)
     } else {
       if (!(is.numeric(restr_tau) && restr_tau >0)) {stop(paste("restr_tau must be numeric and positive, Inf or 'simult_conf'. But restr_tau=", restr_tau, sep=""))}
-      tau = seq(0.2*restr_tau,0.5*restr_tau, length.out = 3)
+      tau = seq(0.2*restr_tau,0.9*restr_tau, length.out = 3)
     }
-    init_grid <- expand.grid(a = c(0.5, 1,1.7, 2.5),                         ### a = distance btw. upper and lower bound \in (0,\infty)]
-                             vmin = c(0.01, 0.1),                    ### vmin = mean drift rate in first condition \in (0,\infty)]
-                             vmax = c(1.4, 2.5, 3.7, 5),                     ### vmax = mean drift rate in last condition \in (\vmin,\infty)]
-                             sv = c(0.01, 0.8, 1.5),                      ### sv = SD of drift rate (normal distr.) \in (0,\infty)]
+    init_grid <- expand.grid(a = c(0.5, 1,1.7, 2.5),                  ### a = distance btw. upper and lower bound \in (0,\infty)]
+                             vmin = c(0.01, 0.1),                     ### vmin = mean drift rate in first condition \in (0,\infty)]
+                             vmax = c(1.4, 2.5, 3.7, 5),              ### vmax = mean drift rate in last condition \in (\vmin,\infty)]
+                             sv = c(0.01, 0.8, 1.5),                  ### sv = SD of drift rate (normal distr.) \in (0,\infty)]
                              z = sum((df$response==1)*df$n)/sum(df$n),### z = mean start point (bias) \in [0,1]
-                             sz = c(0.1),                            ### sz = range of possible start points (unif ditr.; in units of a-z) \in [0,1]
-                             t0 = c(max(mint0-0.2, 0.05), max(min(mint0-0.1,0.2),mint0/2)), ### t0 = minimal motor time \in (0,\infty)]
-                             st0 = c(0.1,  0.2),                     ### st0 = range of possible motor times (unif. distr.) \in [0, t0/2]
-                             # theta0 = seq(-2.5, .45,length.out = 5),  ### theta0 = lowest threshold for confidence rating (in difference from threshold / a)
-                             # thetamax = seq(0.5, 5.5,length.out = 4),    ### thetamax = highest threshold for confidence rating (in distance from threshold / a)
-                             tau = tau,                              ### tau = confidence rating time
-                             svis = seq(0.01, 0.5, length.out = 2),      ### svis = variability in visibility accumulation process
-                             w = seq(0.3, 0.7, length.out = 3),      ### w = weight bewtween evidence and visibility for confidence judgement
-                             sigvis = seq(0.01, 1, length.out = 3))
-  }
-  if (simult_conf) {
-    init_grid <- init_grid[init_grid$t0+init_grid$tau <= mint0, ]
+                             sz = c(0.1),                             ### sz = range of possible start points (unif ditr.; in units of a-z) \in [0,1]
+                             t0 = c(0.05, 0.2),                       ### t0 = proportion of minimal motor time of minimal total response time \in [0,1)
+                             st0 = c(0.1,  0.2),                      ### st0 = range of possible motor times (unif. distr.) \in [0, t0/2]
+                             tau = tau,                               ### tau = post-decisional accumulation time (between 0 and 1, if tau has an upper bound)
+                             svis = seq(0.01, 0.5, length.out = 2),   ### svis = variability in visibility accumulation process
+                             w = seq(0.3, 0.7, length.out = 3),       ### w = weight bewtween evidence and visibility for confidence judgement
+                             sigvis = seq(0.01, 1, length.out = 3),   ### sigivis = between trial variability in drift rate of the visibility process
+                             lambda = c(0, 0.5, 1, 2))                ### lambda = exponent of accumulation time in the denominator of the confidence variable
   }
   # Remove columns for fixed parameters
   init_grid <- init_grid[setdiff(names(init_grid), names(fixed))]
   init_grid <- unique(init_grid)
 
-  ### If no grid-search is desired use mean of possible parameters #
+  ### If no grid-search is desired use mean of possible parameters
   if (!grid_search) {
     init_grid <- as.data.frame(as.list(colMeans(init_grid)))
   }
@@ -68,52 +64,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
   }
   ## Guess suitable confidence thresholds from theoretical distribution of
   ## the confidence measure and proportion of ratings in the data
-
-  conf_probs <- cumsum(table(df$rating))
-  conf_probs <- conf_probs[1:(nRatings-1)]/conf_probs[nRatings]
-  df$correct <- as.numeric(df$stimulus == df$response)
-  MRT <- aggregate(rt~condition+correct, df, mean)
-  p_corrects <- aggregate(correct~condition, df, mean)[['correct']]
-
-  get_start_thetas <- function(paramRow) {
-    paramRow <- c(paramRow, unlist(fixed, use.names = TRUE))
-    v <- c(t(paramRow[paste("v", 1:nConds, sep="")]))
-    MRT_corr <- (filter(MRT, .data$correct==1)$rt-paramRow['t0']-paramRow['st0']/2)
-    MRT_false <- (filter(MRT, .data$correct==0)$rt-paramRow['t0']-paramRow['st0']/2)
-
-    if (simult_conf) {
-      MRT_corr <- MRT_corr -paramRow['tau']
-      MRT_false <- MRT_false -paramRow['tau']
-    }
-    Mconf_corr = paramRow['w']* paramRow['tau']*
-      (-paramRow['a']*paramRow['z']*paramRow['sv']^2+v)/(1+paramRow['sv']^2*MRT_corr)+
-      (1-paramRow['w'])*(MRT_corr+paramRow['tau'])*abs(v)
-
-    Mconf_false = -paramRow['w']*paramRow['tau']*(-paramRow['a']*paramRow['z']*paramRow['sv']^2+v)/
-      (1+paramRow['sv']^2*MRT_false)+
-      (1-paramRow['w'])*(MRT_false+paramRow['tau'])*abs(v)
-
-    VRconf = (paramRow['w']^2*
-                (paramRow['tau'] +
-                   ((paramRow['sv']^2*paramRow['tau']^2)/
-                      (1+paramRow['sv']^2*sum(p_corrects*MRT_corr+(1-p_corrects)*MRT_false)/nConds)))) +
-      ((1-paramRow['w'])^2*
-         (paramRow['sigvis']^2*(sum(p_corrects*MRT_corr+(1-p_corrects)*MRT_false)/nConds+paramRow['tau'])^2 +
-            paramRow['svis']^2*(sum(p_corrects*MRT_corr+(1-p_corrects)*MRT_false)/nConds+paramRow['tau'])))
-
-
-    mixcdf <- function(conf) 1/nConds * sum(p_corrects* pnorm(conf, mean=Mconf_corr, sd=sqrt(VRconf))+
-                                              (1-p_corrects)*pnorm(conf, mean=Mconf_false, sd=sqrt(VRconf)))
-    thetas <- NULL
-    for (i in 1:length(conf_probs)) {
-      thetas[i] <- optimize(function(conf) (mixcdf(conf)-conf_probs[i])^2,
-                            lower=min(Mconf_false,Mconf_corr)- 4*VRconf,
-                            upper=max(Mconf_false,Mconf_corr)+ 4*VRconf)$minimum
-    }
-    c(thetas[1],diff(thetas))
-  }
-  init_thetas <- apply(init_grid, FUN=get_start_thetas, MARGIN=1) # , simplify = TRUE
-  init_thetas <- t(init_thetas)
+  init_thetas <- get_thetas_for_init_grid_dynWEV_simulations(init_grid, df, nRatings, simult_conf, fixed, mint0)
 
 
   #### 1.1. For Nelder-Mead transform all parameters to real values ####
@@ -130,7 +81,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
     if (!("sv" %in% names(fixed))) inits <-  cbind(inits, log(init_grid$sv)) # one SV (SD of drift rate) for all the different conditions
     if (!("z" %in% names(fixed))) inits <- cbind(inits, qnorm(init_grid$z))
     if (!("sz" %in% names(fixed))) inits <- cbind(inits, qnorm(init_grid$sz))
-    if (!("t0" %in% names(fixed))) inits <- cbind(inits, log(init_grid$t0))
+    if (!("t0" %in% names(fixed))) inits <- cbind(inits, qnorm(init_grid$t0))
     if (!("st0" %in% names(fixed))) inits <- cbind(inits, log(init_grid$st0))
     if (!("svis" %in% names(fixed))) inits <- cbind(inits, log(init_grid$svis))
     if (!("sigvis" %in% names(fixed))) inits <- cbind(inits, log(init_grid$sigvis))
@@ -142,12 +93,13 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
         inits <- cbind(inits, qnorm(init_grid$tau / restr_tau))
       }
     }
+    if (!("lambda" %in% names(fixed))) inits <- cbind(inits, log(init_grid$lambda))
     inits <- cbind(inits, init_thetas[,1])
     if (nRatings > 2) {
       inits <- cbind(inits, log(init_thetas[,-1]))
     }
     if (!sym_thetas) {
-      inits <- cbind(inits, init_grid$theta0)
+      inits <- cbind(inits, init_thetas[,1])
       if (nRatings > 2) {
         inits <- cbind(inits, log(init_thetas[,-1]))
       }
@@ -161,7 +113,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
     inits[inits==Inf]<- 1e6
     inits[inits==-Inf]<- -1e6
     parnames <- c(paste("v", 1:nConds, sep=""), 'a', 'sv', 'z', 'sz', 't0', 'st0',
-                   'svis','sigvis', 'w', 'tau', cols_theta)
+                   'svis','sigvis', 'w', 'tau', 'lambda', cols_theta)
     names(inits) <- setdiff(parnames, names(fixed))
 
   } else {
@@ -188,7 +140,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       }
     }
     parnames <- c('a', 'z', 'sz', paste("v", 1:nConds, sep=""),
-                   'st0', 'sv', 't0', cols_theta,'tau', 'w', 'svis', 'sigvis')
+                   'st0', 'sv', 't0', cols_theta,'tau', 'w', 'svis', 'sigvis', 'lambda')
     inits <- init_grid[, setdiff(parnames, names(fixed))]
 
   }
@@ -202,7 +154,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       n.cores <- detectCores()-1
     }
     cl <- makeCluster(type="SOCK", n.cores)
-    clusterExport(cl, c("df", "restr_tau",
+    clusterExport(cl, c("df", "restr_tau", "mint0",
                         "nConds","nRatings", "fixed", "simult_conf", "sym_thetas", "precision"), envir = environment())
   }
 
@@ -221,26 +173,26 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
         logL <-
           parApply(cl, inits, MARGIN=1,
                    function(p) try(neglikelihood_dynWEV_free(p, df,
-                                                             restr_tau, nConds, nRatings, fixed, simult_conf, sym_thetas, precision),
+                                                             restr_tau, nConds, nRatings, fixed, mint0, simult_conf, sym_thetas, precision),
                                    silent=TRUE))
         #stopCluster(cl)
       } else {
         logL <-
           apply(inits, MARGIN = 1,
-                function(p) try(neglikelihood_dynWEV_free(p, df, restr_tau, nConds, nRatings, fixed, simult_conf, sym_thetas, precision),
+                function(p) try(neglikelihood_dynWEV_free(p, df, restr_tau, nConds, nRatings, fixed, mint0, simult_conf, sym_thetas, precision),
                                 silent = TRUE))
       }
     } else {
       if (useparallel) {
         logL <-
           parApply(cl, inits, MARGIN=1,
-                   function(p) try(neglikelihood_dynWEV_bounded(p, df, restr_tau, nConds, nRatings, fixed, simult_conf, sym_thetas, precision),
+                   function(p) try(neglikelihood_dynWEV_bounded(p, df, restr_tau, nConds, nRatings, fixed, mint0, simult_conf, sym_thetas, precision),
                                    silent=TRUE))
         #stopCluster(cl)
       } else {
         logL <-
           apply(inits, MARGIN = 1,
-                function(p) try(neglikelihood_dynWEV_bounded(p, df, restr_tau, nConds, nRatings, fixed, simult_conf, sym_thetas, precision),
+                function(p) try(neglikelihood_dynWEV_bounded(p, df, restr_tau, nConds, nRatings, fixed, mint0, simult_conf, sym_thetas, precision),
                                 silent=TRUE))
       }
     }
@@ -254,9 +206,9 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
   }
 
   if (optim_method!="Nelder-Mead") {
-                      # a,  z, sz,  v1, v2,....,,   st0, sv, t0,  thetaLower1, dthetaLower2.., thetaUpper1... (or theta1,...),  tau, w, svis, sigvis
-    lower_optbound <- c(0,  0,  0,  rep(0, nConds), 0,   0,  0,   rep(c(-Inf,  rep(0, nRatings-2)), 2-as.numeric(sym_thetas)),    0, 0,  0,   0)[!(parnames %in% names(fixed))]
-    upper_optbound <- c(Inf,1,  1,  rep(Inf,nConds),Inf, Inf,Inf, rep(Inf, (2-as.numeric(sym_thetas))*(nRatings-1)),      restr_tau, 1, Inf, Inf)[!(parnames %in% names(fixed))]
+                      # a,  z, sz,  v1, v2,....,,   st0, sv, t0,thetaLower1, dthetaLower2.., thetaUpper1... (or theta1,...),  tau, w, svis, sigvis, lambda
+    lower_optbound <- c(0,  0,  0,  rep(0, nConds), 0,   0,  0, rep(c(-Inf,  rep(0, nRatings-2)), 2-as.numeric(sym_thetas)),    0, 0,  0,   0,      0)[!(parnames %in% names(fixed))]
+    upper_optbound <- c(Inf,1,  1,  rep(Inf,nConds),Inf, Inf,1, rep(Inf, (2-as.numeric(sym_thetas))*(nRatings-1)),      restr_tau, 1, Inf, Inf,     Inf)[!(parnames %in% names(fixed))]
   }
 
 
@@ -272,13 +224,13 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       start <- c(t(inits[i,]))
       names(start) <- names(inits)
       for (l in 1:opts$nRestarts){
-        start <- start + rnorm(length(start), sd=t(t(start))/20)
-
+        start <- start + rnorm(length(start), sd=pmax(0.001, abs(t(t(start))/20)))
         if (optim_method == "Nelder-Mead") {
           try(m <- optim(par = start,
                          fn = neglikelihood_dynWEV_free,
                          data=df, restr_tau = restr_tau, nConds=nConds, nRatings=nRatings,
-                         fixed=fixed, simult_conf=simult_conf, sym_thetas=sym_thetas, precision=precision,
+                         fixed=fixed, mint0=mint0, simult_conf=simult_conf,
+                         sym_thetas=sym_thetas, precision=precision,
                          method="Nelder-Mead",
                          control = list(maxit = opts$maxit, reltol = opts$reltol)))
         } else if (optim_method =="bobyqa") {
@@ -287,7 +239,8 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
                           fn = neglikelihood_dynWEV_bounded,
                           lower = lower_optbound, upper = upper_optbound,
                           data=df, restr_tau = restr_tau, nConds=nConds, nRatings=nRatings,
-                          fixed=fixed, simult_conf=simult_conf, sym_thetas=sym_thetas, precision=precision,
+                          fixed=fixed, mint0=mint0, simult_conf=simult_conf,
+                          sym_thetas=sym_thetas, precision=precision,
                           control = list(maxfun=opts$maxfun,
                                          rhobeg = min(0.2, 0.2*restr_tau, 0.2*max(abs(start))),
                                          npt = length(start)+5)))
@@ -305,7 +258,8 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
                          fn = neglikelihood_dynWEV_bounded,
                          lower = lower_optbound, upper = upper_optbound,
                          data=df,  restr_tau = restr_tau, nConds=nConds, nRatings=nRatings,
-                         fixed=fixed, simult_conf=simult_conf, sym_thetas=sym_thetas, precision=precision,
+                         fixed=fixed, mint0=mint0, simult_conf=simult_conf,
+                         sym_thetas=sym_thetas, precision=precision,
                          method="L-BFGS-B",
                          control = list(maxit = opts$maxit, factr = opts$factr)))
         } else {
@@ -354,25 +308,26 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       start <- c(t(start))
       names(start) <- parnames
       for (l in 1:opts$nRestarts){
-        start <- start + rnorm(length(start), sd=t(t(start))/20)
+        start <- start + rnorm(length(start), sd=pmax(0.001, abs(t(t(start))/20)))
         if (optim_method == "Nelder-Mead") {
-          try(m <- optim(par = start,
+          m <- try(optim(par = start,
                          fn = neglikelihood_dynWEV_free,
                          data=df,  restr_tau = restr_tau, nConds=nConds, nRatings=nRatings,
-                         fixed=fixed, simult_conf=simult_conf, sym_thetas=sym_thetas, precision=precision,
+                         fixed=fixed, mint0=mint0, simult_conf=simult_conf,
+                         sym_thetas=sym_thetas, precision=precision,
                          method="Nelder-Mead",
                          control = list(maxit = opts$maxit, reltol = opts$reltol)))
         } else if (optim_method =="bobyqa") {
           start <- pmax(pmin(start, upper_optbound-1e-6), lower_optbound+1e-6)
-          try(m <- bobyqa(par = start,
+          m <- try(bobyqa(par = start,
                           fn = neglikelihood_dynWEV_bounded,
                           lower = lower_optbound, upper = upper_optbound,
                           data=df,  restr_tau = restr_tau, nConds=nConds, nRatings=nRatings,
-                          fixed=fixed, simult_conf=simult_conf, sym_thetas=sym_thetas, precision=precision,
+                          fixed=fixed, mint0=mint0, simult_conf=simult_conf,
+                          sym_thetas=sym_thetas, precision=precision,
                           control = list(maxfun=opts$maxfun,
                                          rhobeg = min(0.2, 0.2*restr_tau, 0.2*max(abs(start))),
-                                         npt = length(start)+5)),
-              silent=TRUE)
+                                         npt = length(start)+5)))
           ## rhobeg should be: about 0.1*(greatest expected change in parameters --> <= 1-2 (for a, thetas or v's) )
           ##                   smaller than min(abs(upper-lower)) = min(1, restr_tau)
           ##                   --> so we use min(0.2*restr_tau, 0.2, 0.2*max(abs(par))).
@@ -387,7 +342,8 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
                          fn = neglikelihood_dynWEV_bounded,
                          lower = lower_optbound, upper = upper_optbound,
                          data=df,  restr_tau = restr_tau, nConds=nConds, nRatings=nRatings,
-                         fixed=fixed, simult_conf=simult_conf, sym_thetas=sym_thetas, precision=precision,
+                         fixed=fixed, mint0=mint0, simult_conf=simult_conf,
+                         sym_thetas=sym_thetas, precision=precision,
                          method="L-BFGS-B",
                          control = list(maxit = opts$maxit, factr = opts$factr)))
         } else {
@@ -412,7 +368,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       if (exists("fit") && is.list(fit)){
         return(c(fit$value,fit$par))
       } else {
-        return(rep(NA, length(start)+1))
+        return(c(m[1] , rep(NA, length(start))))
       } # end of node-function
     }
     clusterExport(cl, c("parnames", "opts", "optim_method","optim_node" ), envir = environment())
@@ -435,21 +391,19 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
     p <- fit$par
     if (optim_method=="Nelder-Mead") {
       res[,paste("v",1:(nConds), sep="")] <- exp(p[1:(nConds)])
+      if (length(fixed)>=1) res <- cbind(res, as.data.frame(fixed))
+
       if (!("a" %in% names(fixed))) res$a <- exp(p[["a"]])
       if (!("sv" %in% names(fixed))) res$sv <- exp(p[["sv"]])
       if (!("z" %in% names(fixed))) res$z <- pnorm(p[["z"]]) ## relative mean starting point
-      if (!("sz" %in% names(fixed))) {
-        if (!("z" %in% names(fixed))) {
-          res$sz <- (min(res$z, (1-res$z))*2)*pnorm(p[["sz"]]) ##
-        } else {
-          res$sz <- (min(fixed[['z']], (1-fixed[['z']]))*2)*pnorm(p[["sz"]]) ##
-        }
-      }
-      if (!("t0" %in% names(fixed))) res$t0 <- exp(p[["t0"]])
+      if (!("sz" %in% names(fixed))) res$sz <- (min(res$z, (1-res$z))*2)*pnorm(p[["sz"]])
+      if (!("t0" %in% names(fixed))) res$t0 <- pnorm(p[["t0"]])*mint0
       if (!("st0" %in% names(fixed))) res$st0 <- exp(p[["st0"]])
       if (!("tau" %in% names(fixed))) {
         if (restr_tau == Inf) {
           res$tau <- exp(p[["tau"]])
+        } else if (simult_conf) {
+          res$tau <- pnorm(p[["tau"]])*(mint0-res$t0)
         } else {
           res$tau <- restr_tau * pnorm(p[["tau"]])
         }
@@ -457,6 +411,7 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       if (!("svis" %in% names(fixed))) res$svis <- exp(p[["svis"]])
       if (!("sigvis" %in% names(fixed))) res$sigvis <- exp(p[["sigvis"]])
       if (!("w" %in% names(fixed))) res$w <- pnorm(p[["w"]])
+      if (!("lambda" %in% names(fixed))) res$lambda <- exp(p[["lambda"]])
       if (nRatings > 2) {
         if (sym_thetas) {
           res[,paste("theta",1:(nRatings-1), sep="")] <- cumsum(c(p[["theta1"]], exp(p[paste0("dtheta", 2:(nRatings-1))])))
@@ -484,14 +439,11 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
         }
         p <- p[ -grep(names(p), pattern="dtheta")]
       }
-      if (!("sz" %in% names(fixed))) {
-        #p["sz"] <- min(0.999999, p["sz"])
-        if (!("z" %in% names(fixed))) {
-          p['sz'] <- (min(p['z'], (1-p['z']))*2)*p["sz"] ##
-        } else {
-          p['sz'] <- (min(fixed[['z']], (1-fixed[['z']]))*2)*p["sz"] ##
-        }
-      }
+      if (length(fixed)>=1) p <- c(p, unlist(fixed))
+      if (!("t0" %in% names(fixed))) p['t0'] <- p['t0']*mint0
+      if (simult_conf & !("tau" %in% names(fixed))) p['tau'] <- p['tau']*(mint0-p[['t0']])
+      if (!("sz" %in% names(fixed))) p['sz'] <- (min(p['z'], (1-p['z']))*2)*p["sz"] ##
+
       res <-   data.frame(matrix(nrow=1, ncol=length(p)))
       res[1,] <- p
       names(res) <- names(p)      # a,  z, sz,v1, v2,....,,   st0, sv, t0, thetaLower1,dthetaLower2-4,   thetaUpper1,dthetaUpper2-4,    tau,       w, svis, sigvis
@@ -505,14 +457,12 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
       k <- ncol(res)
     }
     if (sym_thetas) {
-      parnames <- c(paste("v", 1:nConds, sep=""), 'sv', 'a', 'z', 'sz', 't0','st0', paste("theta", 1:(nRatings-1), sep=""), 'tau', 'w', 'svis','sigvis')
+      parnames <- c(paste("v", 1:nConds, sep=""), 'sv', 'a', 'z', 'sz', 't0','st0', paste("theta", 1:(nRatings-1), sep=""), 'tau', 'w', 'svis','sigvis', 'lambda')
     } else {
-      parnames <- c(paste("v", 1:nConds, sep=""), 'sv', 'a', 'z', 'sz', 't0','st0', paste("thetaLower", 1:(nRatings-1), sep=""), paste("thetaUpper", 1:(nRatings-1), sep=""), 'tau', 'w', 'svis','sigvis')
-    }
-    res <- res[, setdiff(parnames, names(fixed))]
-    if (length(fixed)>=1) {
-      res <- cbind(res, as.data.frame(fixed))
+      parnames <- c(paste("v", 1:nConds, sep=""), 'sv', 'a', 'z', 'sz', 't0','st0', paste("thetaLower", 1:(nRatings-1), sep=""), paste("thetaUpper", 1:(nRatings-1), sep=""), 'tau', 'w', 'svis','sigvis', 'lambda')
     }
+    res <- res[, parnames]
+
     res$fixed <- paste(c("sym_thetas", names(fixed)), c(sym_thetas,fixed), sep="=", collapse = ", ")
     res$negLogLik <- fit$value
     res$N <- N
@@ -532,26 +482,24 @@ fittingdynWEV <- function(df, nConds, nRatings, fixed, sym_thetas,
 
 
 neglikelihood_dynWEV_free <-   function(p, data,
-                                        restr_tau, nConds, nRatings, fixed, simult_conf, sym_thetas, precision=1e-5)
+                                        restr_tau, nConds, nRatings, fixed, mint0, simult_conf, sym_thetas, precision=1e-5)
 {
   # get parameter vector back from real transformations
   paramDf <-  data.frame(matrix(nrow=1, ncol=0))
   paramDf[,paste("v",1:(nConds), sep="")] <- exp(p[1:(nConds)])
+  if (length(fixed)>=1) paramDf <- cbind(paramDf, as.data.frame(fixed))
+
   if (!("a" %in% names(fixed))) paramDf$a <- exp(p[["a"]])
   if (!("sv" %in% names(fixed))) paramDf$sv <- exp(p[["sv"]])
   if (!("z" %in% names(fixed))) paramDf$z <- pnorm(p[["z"]]) ## relative mean starting point
-  if (!("sz" %in% names(fixed))) {
-    if (!("z" %in% names(fixed))) {
-      paramDf$sz <- (min(paramDf$z, (1-paramDf$z))*2)*pnorm(p[["sz"]]) ##
-    } else {
-      paramDf$sz <- (min(fixed[['z']], (1-fixed[['z']]))*2)*pnorm(p[["sz"]]) ##
-    }
-  }
-  if (!("t0" %in% names(fixed))) paramDf$t0 <- exp(p[["t0"]])
+  if (!("sz" %in% names(fixed))) paramDf$sz <- (min(paramDf$z, (1-paramDf$z))*2)*pnorm(p[["sz"]]) ##
+  if (!("t0" %in% names(fixed))) paramDf$t0 <- pnorm(p[["t0"]])*mint0
   if (!("st0" %in% names(fixed))) paramDf$st0 <- exp(p[["st0"]])
   if (!("tau" %in% names(fixed))) {
     if (restr_tau == Inf) {
       paramDf$tau <- exp(p[["tau"]])
+    } else if (simult_conf) {
+      paramDf$tau <- pnorm(p[["tau"]])*(mint0-paramDf$t0)
     } else {
       paramDf$tau <- restr_tau * pnorm(p[["tau"]])
     }
@@ -559,6 +507,7 @@ neglikelihood_dynWEV_free <-   function(p, data,
   if (!("svis" %in% names(fixed))) paramDf$svis <- exp(p[["svis"]])
   if (!("sigvis" %in% names(fixed))) paramDf$sigvis <- exp(p[["sigvis"]])
   if (!("w" %in% names(fixed))) paramDf$w <- pnorm(p[["w"]])
+  if (!("lambda" %in% names(fixed))) paramDf$lambda <- exp(p[["lambda"]])
 
 
   if (nRatings > 2) {
@@ -581,9 +530,6 @@ neglikelihood_dynWEV_free <-   function(p, data,
     return(1e12)
   }
 
-  if (length(fixed)>=1) {
-    paramDf <- cbind(paramDf, as.data.frame(fixed))
-  }
   negloglik <- -LogLikWEV(data, paramDf, "dynWEV", simult_conf, precision, stop_on_error=FALSE)
   return(negloglik)
 }
@@ -591,7 +537,7 @@ neglikelihood_dynWEV_free <-   function(p, data,
 
 
 neglikelihood_dynWEV_bounded <-   function(p, data,
-                                           restr_tau, nConds, nRatings, fixed, simult_conf, sym_thetas=FALSE, precision=1e-5)
+                                           restr_tau, nConds, nRatings, fixed, mint0, simult_conf, sym_thetas=FALSE, precision=1e-5)
 {
   # get parameter vector back from real transformations
   paramDf <-   data.frame(matrix(nrow=1, ncol=length(p)))
@@ -609,8 +555,11 @@ neglikelihood_dynWEV_bounded <-   function(p, data,
   if (length(fixed)>=1) {
     paramDf <- cbind(paramDf, as.data.frame(fixed))
   }
-  paramDf['sz'] <- (min(paramDf['z'], 1-paramDf['z'])*2)*paramDf['sz']
-
+  if (!("sz" %in% names(fixed))) paramDf['sz'] <- (min(paramDf['z'], 1-paramDf['z'])*2)*paramDf['sz']
+  if (!("t0" %in% names(fixed))) paramDf['t0'] <- paramDf['t0']*mint0
+  if (simult_conf & !("tau" %in% names(fixed))) {
+    paramDf['tau'] <- paramDf['tau']*(mint0-paramDf['t0'])
+  }
   negloglik <- -LogLikWEV(data, paramDf, "dynWEV", simult_conf, precision, stop_on_error=FALSE)
   return(negloglik)
 }
