@@ -22,120 +22,116 @@ using namespace Rcpp;
 #define EPSILON 1e-6
 
 // Forward declarations
-double g_minus_2DSD (double t);
-double g_plus_2DSD  (double t);
+double g_minus_2DSD (double t, NumericVector params);
 
-static double integral_t0_g_minus_2DSD (double t, Parameters *params);
-static double integral_z_g_minus_2DSD  (double t, Parameters *params);
-static double integral_v_g_minus_2DSD  (double t, double zr, Parameters *params);
+static double integral_t0_g_minus_2DSD (double t, NumericVector params);
+static double integral_z_g_minus_2DSD  (double t, NumericVector params);
+static double integral_v_g_minus_2DSD  (double t, double zr, NumericVector params);
 
-static double g_minus_no_var_2DSD     (double t, double a, double zr, double v, double tau, double th1, double th2, double omega);
+static double g_minus_no_var_2DSD     (double t, double a, double zr, double v, double tau, double th1, double th2, double lambda);
 static double g_minus_small_time_2DSD (double t, double zr, int N);
 static double g_minus_large_time_2DSD (double t, double zr, int N);
 
 // TODO: Make sure these function names are accurate
-static double integrate_z_over_t_2DSD  (Parameters *params, double a, double b, double step_width);
-static double integrate_v_over_zr_2DSD (Parameters *params, double a, double b, double t, double step_width);
+static double integrate_z_over_t_2DSD  (NumericVector params, double a, double b, double step_width);
+static double integrate_v_over_zr_2DSD (NumericVector params, double a, double b, double t, double step_width);
 
 
 // Main calls
-NumericVector density_2DSD (NumericVector rts, int boundary, int stopon0)
+NumericVector density_2DSD (NumericVector rts, NumericVector params, int boundary, int stopon0)
 {
     int length = rts.length();
     NumericVector out(length);
     if (stopon0==1) {
       if (boundary == 1) {
+        params[7] = 1- params[7]; // z -> 1 - z
+        params[1] = - params[1]; // v  -> - v
+        params[3] = - params[3]; // d  -> - d
         for (int i = 0; i < length; i++) {
-          out[i] =  g_plus_2DSD(rts[i]);
+          out[i] =  g_minus_2DSD(rts[i], params);
           if (out[i]==0) break;
-          }
-        } // Calc upper
+        }
+      } // Calc upper
       else {
         for (int i = 0; i < length; i++) {
-          out[i] = -g_minus_2DSD(rts[i]);
+          out[i] = -g_minus_2DSD(rts[i], params);
           if (out[i]==0) break;
           }
         } // Calc lower
     } else {
-      if (boundary == 1) { for (int i = 0; i < length; i++) { out[i] =  g_plus_2DSD(rts[i]);  } } // Calc upper
-      else { for (int i = 0; i < length; i++) { out[i] = -g_minus_2DSD(rts[i]); } } // Calc lower
+      if (boundary == 1) {
+        params[7] = 1- params[7]; // z -> 1 - z
+        params[1] = - params[1]; // v  -> - v
+        params[3] = - params[3]; // d  -> - d
+        for (int i = 0; i < length; i++) {
+          out[i] =  g_minus_2DSD(rts[i], params);
+        }
+      } // Calc upper
+      else { for (int i = 0; i < length; i++) { out[i] = -g_minus_2DSD(rts[i], params); } } // Calc lower
     }
 
 
     return out;
 }
 
-double g_minus_2DSD(double t)
+double g_minus_2DSD(double t, NumericVector params)
 {
-    return integral_t0_g_minus_2DSD (t - g_Params->t0 - 0.5*g_Params->d, g_Params);
-}
-
-double g_plus_2DSD(double t)
-{
-    // Make a copy so we don't disturb our params
-    // (?TODO: we could optimise the object creation out and just set them back after the call)
-    Parameters new_params(*g_Params);
-    new_params.zr = 1 - g_Params->zr;
-    new_params.v = -g_Params->v;
-    //new_params.th1 =  -g_Params->th2;
-    //new_params.th2 =  -g_Params->th1;
-
-    return integral_t0_g_minus_2DSD (t - new_params.t0 + 0.5*new_params.d, &new_params);
+    return integral_t0_g_minus_2DSD (t - params[2] - 0.5*params[3], params);
 }
 
 
 
-static double integral_t0_g_minus_2DSD (double t, Parameters *params)
+static double integral_t0_g_minus_2DSD (double t, NumericVector params)
 {
     double res;
 
-    if (params->st0 < params->TUNE_ST0_EPSILON) // 170501   was == 0)
+    if (params[6] < params[15]) // 170501   was == 0)
     {
         res = integral_z_g_minus_2DSD (t, params);
     }
     else
     {
         res = integrate_z_over_t_2DSD(params,
-                        t - .5*params->st0,
-                        t + .5*params->st0, params->TUNE_INT_T0) / params->st0;
+                        t - .5*params[6],
+                        t + .5*params[6], params[12]) / params[6];
     }
 
     return res;
 }
 
 
-static double integral_z_g_minus_2DSD (double t, Parameters *params)
+static double integral_z_g_minus_2DSD (double t, NumericVector params)
 {
     double res;
 
     if (t <= 0) return 0;
 
-    if (params->szr < params->TUNE_SZ_EPSILON)
+    if (params[4] < params[14])
     {
-        res = integral_v_g_minus_2DSD (t, params->zr, params);
+        res = integral_v_g_minus_2DSD (t, params[7], params);
     }
     else
     {
-        res = integrate_v_over_zr_2DSD(params, params->zr - .5*params->szr, params->zr + .5*params->szr,
-                                  t, params->TUNE_INT_Z) / params->szr;
+        res = integrate_v_over_zr_2DSD(params, params[7] - .5*params[4], params[7] + .5*params[4],
+                                  t, params[13]) / params[4];
     }
     return res;
 }
 
 
-static double integral_v_g_minus_2DSD (double t, double zr, Parameters *params)
+static double integral_v_g_minus_2DSD (double t, double zr, NumericVector params)
 {
-    double a = params->a;
-    double v = params->v;
-    double sv = params->sv;
-    double tau = params->tau;
-    double th2 = params->th2;
-    double th1 = params->th1;
-    double omega = params->omega;
+    double a = params[0];
+    double v = params[1];
+    double sv = params[5];
+    double tau = params[8];
+    double th2 = params[10];
+    double th1 = params[9];
+    double lambda = params[11];
 
-    if (params->sv == 0)
+    if (sv == 0)
     {
-        return g_minus_no_var_2DSD(t, a, zr, v, tau, th1, th2, omega);
+        return g_minus_no_var_2DSD(t, a, zr, v, tau, th1, th2, lambda);
     }
 
     int N_small, N_large;
@@ -148,12 +144,12 @@ static double integral_v_g_minus_2DSD (double t, double zr, Parameters *params)
 
     mean_conf = - (tau*v - a*zr*(sv*sv*(tau+t)+1)) / (sv2t); // ((v - sv*sv*a*zr)*tau) / sv2t;
     sd_conf = sqrt( tau*(sv*sv*tau+sv2t)/(sv2t));
-    diff_normal = 0.5*(erf((th2*pow(t+tau, omega) - mean_conf) / (M_SQRT2 * sd_conf)) -
-                        erf((th1*pow(t+tau, omega) - mean_conf) / (M_SQRT2 * sd_conf)));
+    diff_normal = 0.5*(erf((th2*pow(t+tau, lambda) - mean_conf) / (M_SQRT2 * sd_conf)) -
+                        erf((th1*pow(t+tau, lambda) - mean_conf) / (M_SQRT2 * sd_conf)));
 
     // factor = exp(-a*zr*v - 0.5*v*v*t) / (a*a); //sqrt(t+tau)
-    // diff_normal = 0.5*(erf((th2*pow(t+tau, omega) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))) -
-    //   erf((th1*pow(t+tau, omega) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))));
+    // diff_normal = 0.5*(erf((th2*pow(t+tau, lambda) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))) -
+    //   erf((th1*pow(t+tau, lambda) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))));
     //
     //
 
@@ -188,15 +184,15 @@ static double integral_v_g_minus_2DSD (double t, double zr, Parameters *params)
 
 
 static double g_minus_no_var_2DSD(double t, double a, double zr, double v,
-                                   double tau, double th1, double th2, double omega)
+                                   double tau, double th1, double th2, double lambda)
 {
     int N_small, N_large;
     double diff_normal, simple, factor, eps;
     double ta = t/(a*a);
 
     factor = exp(-a*zr*v - 0.5*v*v*t) / (a*a); //sqrt(t+tau)
-    diff_normal = 0.5*(erf((th2*pow(t+tau, omega) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))) -
-      erf((th1*pow(t+tau, omega) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))));
+    diff_normal = 0.5*(erf((th2*pow(t+tau, lambda) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))) -
+      erf((th1*pow(t+tau, lambda) + (tau*v - a*zr)) / (M_SQRT2 * sqrt(tau))));
 
     /*
     if (std::isinf(factor))
@@ -270,7 +266,7 @@ static double g_minus_large_time_2DSD(double t, double zr, int N)
 
 // CONVERSION NOTE: Simplest way to deal with the integrate function is to remove
 //                  the clever recursiveness and instead (ugh) duplicate code
-static double integrate_z_over_t_2DSD (Parameters *params, double a, double b, double step_width)
+static double integrate_z_over_t_2DSD (NumericVector params, double a, double b, double step_width)
 {
     double width = b-a;
     int N = std::max(4, (int) (width / step_width));
@@ -285,7 +281,7 @@ static double integrate_z_over_t_2DSD (Parameters *params, double a, double b, d
     return result;
 }
 
-static double integrate_v_over_zr_2DSD (Parameters *params, double a, double b, double t, double step_width)
+static double integrate_v_over_zr_2DSD (NumericVector params, double a, double b, double t, double step_width)
 {
     double width = b-a;
     int N = std::max(4, (int) (width / step_width));
