@@ -31,10 +31,12 @@ using namespace Rcpp;
 
 // Used by both PDF and CDF (also includes tuning params) also by 2DSD and WEV implementation
 #include "Parameters.h"
+#include "ValidateParameters.h"
+
 
 // While not enforced, this is the global parameters Singleton
 //   To be created and freed in the other calls in SeqSampConf.cpp
-Parameters *g_Params;
+//Parameters *g_Params;
 
 #define BOUNDARY_LOWER 0
 #define BOUNDARY_UPPER 1
